#!/bin/bash
export LC_ALL=C 


if [ "${#}" != "1" ]
then
  echo "Usage: ${0} file.bin"
  echo
  exit 1
fi

if [ ! -f "${1}" ]
then
  echo "File \"${1}\" not found."
  echo
  exit 1
fi

if [ ! -s "${1}" ]
then
  echo "File \"${1}\" is empty."
  echo
  exit 1
fi

size=$(stat -c%s "${1}")
size=$(( $size + 0 ))
if [ $size -gt 25000 ]
then
  echo "File \"${1}\" is too large ($size > 25,000)."
  echo
  exit 1
fi



write_first () {

  cat << EOFIRST
/*

                                   _JNJ\`                        
                               .JNMH\`                           
                              JMMF\`       \`;.                   
                            .NMM)          \`MN.                 
                            MMM)            (MML                
                           (MMM\`             MMML               
                     M  I  D  N  I  G  H  T    C  o  D  E       
                           (NMMF             MHNH               
                            NMML            .MMM                
                             NMML          .NMH                 
                              4MMNL       .#F                   
                               \`4HNNL_   \`                      
                                   \`"""\`                        

                          Copyright (C) 2004-2015
       "Ian (Larry) Latter" <ian dot latter at midnightcode dot org>

    Midnight Code is a registered trademark of Ian Latter.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, and mirrored at the Midnight Code web
    site; as at version 2 of the License only.
  
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    General Public License for more details.
  
    You should have received a copy of the GNU General Public License
    (version 2) along with this program; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
    02111-1307 USA, or see http://midnightcode.org/gplv2.txt

*/

#include <avr/pgmspace.h>

EOFIRST
}


write_next () {
  size=$(stat -c%s "${1}")
  printf "int with_light = 1;\n"
  printf "int with_sound = 1;\n"
  printf "int start_delay = 10;\n"
  printf "\n"
  printf "PROGMEM prog_uchar const file_bytes[%d]={\n" "${size}"
}


write_last () {

  cat << EOLAST
};

int led = 13;
int piezo = 11;

void type_byte(unsigned char b) {
  Keyboard.write(b);
}

void piezo_beep() {
  int frequency = 4000;
  for(int i = 0; i < ((frequency / 1000) * 400); i++) {
    digitalWrite(piezo, HIGH);
    delayMicroseconds(500000 / frequency);
    digitalWrite(piezo, LOW);
    delayMicroseconds(500000 / frequency);
  }
}

void user_notify(unsigned int count) {
  for(int i = 0; i < count; i++) {
    if(with_light) {
      digitalWrite(led, HIGH);
    }
    if(with_sound) {
      piezo_beep();
    }
    if(with_light) {
      delay(250);
      digitalWrite(led, LOW);
      delay(250);
    }
  }
}

void setup() {
  pinMode(led, OUTPUT);
  Keyboard.begin();
  delay(start_delay * 1000);
  user_notify(3);
  delay(750);
  for(int i = 0; i < sizeof(file_bytes); i++) {
    type_byte(pgm_read_byte_near(file_bytes + i));
  }
  delay(500);
  user_notify(1);
  delay(1000);
}

void loop() {

}
EOLAST
}


write_first
write_next "${1}"

counter=0
first=0
cat "${1}" | hexdump -ve '1/1 "%.2x"' | while read -n2 char
do
  if [ ${first} != 0 ]
  then
    printf ', '
  fi
  first=1
  if [ $counter -eq 8 ]
  then
    printf '\n'
    counter=0
  fi
  counter=$(( $counter + 1 ))
  if [ $counter -eq 1 ]
  then
    printf '    '
  fi
  printf '0x%s' "$char"
done
printf '\n'

write_last
